@@ -10,15 +10,17 @@ mod logical_matchers;
 mod name;
 mod perm;
 mod printer;
+mod printf;
 mod prune;
+mod regex;
 mod size;
 mod time;
 mod type_matcher;
 
-use regex::Regex;
-use std::error::Error;
+use ::regex::Regex;
 use std::path::Path;
 use std::time::SystemTime;
+use std::{error::Error, str::FromStr};
 use walkdir::DirEntry;
 
 use super::{Config, Dependencies};
@@ -115,7 +117,8 @@ pub fn build_top_level_matcher(
     if !top_level_matcher.has_side_effects() {
         let mut new_and_matcher = logical_matchers::AndMatcherBuilder::new();
         new_and_matcher.new_and_condition(top_level_matcher);
-        new_and_matcher.new_and_condition(printer::Printer::new_box());
+        new_and_matcher
+            .new_and_condition(printer::Printer::new_box(printer::PrintDelimiter::Newline));
         return Ok(new_and_matcher.build());
     }
     Ok(top_level_matcher)
@@ -197,6 +200,8 @@ fn build_matcher_tree(
 ) -> Result<(usize, Box<dyn Matcher>), Box<dyn Error>> {
     let mut top_level_matcher = logical_matchers::ListMatcherBuilder::new();
 
+    let mut regex_type = regex::RegexType::default();
+
     // can't use getopts for a variety or reasons:
     // order of arguments is important
     // arguments can start with + as well as -
@@ -205,7 +210,15 @@ fn build_matcher_tree(
     let mut invert_next_matcher = false;
     while i < args.len() {
         let possible_submatcher = match args[i] {
-            "-print" => Some(printer::Printer::new_box()),
+            "-print" => Some(printer::Printer::new_box(printer::PrintDelimiter::Newline)),
+            "-print0" => Some(printer::Printer::new_box(printer::PrintDelimiter::Null)),
+            "-printf" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+                Some(printf::Printf::new_box(args[i])?)
+            }
             "-true" => Some(logical_matchers::TrueMatcher::new_box()),
             "-false" => Some(logical_matchers::FalseMatcher::new_box()),
             "-name" => {
@@ -222,6 +235,28 @@ fn build_matcher_tree(
                 i += 1;
                 Some(name::CaselessNameMatcher::new_box(args[i])?)
             }
+            "-regextype" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+                regex_type = regex::RegexType::from_str(args[i])?;
+                None
+            }
+            "-regex" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+                Some(regex::RegexMatcher::new_box(regex_type, args[i], false)?)
+            }
+            "-iregex" => {
+                if i >= args.len() - 1 {
+                    return Err(From::from(format!("missing argument to {}", args[i])));
+                }
+                i += 1;
+                Some(regex::RegexMatcher::new_box(regex_type, args[i], true)?)
+            }
             "-type" => {
                 if i >= args.len() - 1 {
                     return Err(From::from(format!("missing argument to {}", args[i])));
@@ -382,6 +417,10 @@ fn build_matcher_tree(
                 config.help_requested = true;
                 None
             }
+            "-version" | "--version" => {
+                config.version_requested = true;
+                None
+            }
 
             _ => return Err(From::from(format!("Unrecognized flag: '{}'", args[i]))),
         };
@@ -419,7 +458,13 @@ mod tests {
     pub fn get_dir_entry_for(directory: &str, filename: &str) -> DirEntry {
         for wrapped_dir_entry in WalkDir::new(fix_up_slashes(directory)) {
             let dir_entry = wrapped_dir_entry.unwrap();
-            if dir_entry.file_name().to_string_lossy() == filename {
+            if dir_entry
+                .path()
+                .strip_prefix(directory)
+                .unwrap()
+                .to_string_lossy()
+                == fix_up_slashes(filename)
+            {
                 return dir_entry;
             }
         }
